package java.util;

import java.util.function.DoubleConsumer;

public class DoubleSummaryStatistics implements DoubleConsumer
{
    private long count;
    private double sum;
    private double sumCompensation;
    private double simpleSum;
    private double min;
    private double max;
    
    public DoubleSummaryStatistics() {
        this.min = Double.POSITIVE_INFINITY;
        this.max = Double.NEGATIVE_INFINITY;
    }
    
    @Override
    public void accept(double n) {
        ++this.count;
        this.simpleSum += n;
        this.sumWithCompensation(n);
        this.min = Math.min(this.min, n);
        this.max = Math.max(this.max, n);
    }
    
    public void combine(DoubleSummaryStatistics doubleSummaryStatistics) {
        this.count += doubleSummaryStatistics.count;
        this.simpleSum += doubleSummaryStatistics.simpleSum;
        this.sumWithCompensation(doubleSummaryStatistics.sum);
        this.sumWithCompensation(doubleSummaryStatistics.sumCompensation);
        this.min = Math.min(this.min, doubleSummaryStatistics.min);
        this.max = Math.max(this.max, doubleSummaryStatistics.max);
    }
    
    private void sumWithCompensation(double n) {
        double n2 = n - this.sumCompensation;
        double sum = this.sum + n2;
        this.sumCompensation = sum - this.sum - n2;
        this.sum = sum;
    }
    
    public final long getCount() {
        return this.count;
    }
    
    public final double getSum() {
        double v = this.sum + this.sumCompensation;
        if (Double.isNaN(v) && Double.isInfinite(this.simpleSum)) {
            return this.simpleSum;
        }
        return v;
    }
    
    public final double getMin() {
        return this.min;
    }
    
    public final double getMax() {
        return this.max;
    }
    
    public final double getAverage() {
        return (this.getCount() > 0L) ? (this.getSum() / (double)this.getCount()) : 0.0;
    }
    
    @Override
    public String toString() {
        return String.format("%s{count=%d, sum=%f, min=%f, average=%f, max=%f}", new Object[] { this.getClass().getSimpleName(), Long.valueOf(this.getCount()), Double.valueOf(this.getSum()), Double.valueOf(this.getMin()), Double.valueOf(this.getAverage()), Double.valueOf(this.getMax()) });
    }
}
